@@ -1,7 +1,8 @@
+"""Module for OC Event API (external API)"""
+
 import os
 import json
-from pprint import pprint
-from typing import List, Optional, Union
+from typing import Generator, List, Optional, Union
 
 import pendulum
 import requests
@@ -18,6 +19,8 @@ from ocpy.model.scheduling import Scheduling
 
 
 class Event:
+    """Class for Opencast events"""
+
     def __init__(self, user, password, url, data):
         self.base_url = url
         self.user = user
@@ -101,7 +104,7 @@ class Event:
 
     def get_publication_status(self):
         return self.data.get("publication_status")
-    
+
     def get_status(self):
         return self.data.get("status")
 
@@ -119,7 +122,9 @@ class Event:
         if res is None:
             url = self.base_url + "/acl"
             res = requests.get(
-                url, timeout=kwargs.pop("timeout", 30), auth=HTTPBasicAuth(self.user, self.password)
+                url,
+                timeout=kwargs.pop("timeout", 30),
+                auth=HTTPBasicAuth(self.user, self.password),
             )
             if res.ok:
                 res = res.json()
@@ -130,116 +135,77 @@ class Event:
             acls.append(ACL(a["allow"], a["role"], Action(a["action"])))
         return acls
 
-    def set_acl(self, acl, **kwargs):
+    def set_acl(self, acl_list, **kwargs):
         url = self.base_url + "/acl"
-        if isinstance(acl, List):
+        if isinstance(acl_list, List):
             acls = []
-            for a in acl:
+            for a in acl_list:
                 if isinstance(a, ACL):
                     acls.append(a.get_acl_dict())
                 else:
                     acls.append(a)
-            acl = acls
-        if isinstance(acl, ACL):
-            acl = acl.get_acl_dict()
-        print(json.dumps(acl))
-        acl = {
-            "allow": True,
-            "role": "ROLE_USER_SG8018_KIT_EDU",
-            "action": "read",
+            acl_list = acls
+        if isinstance(acl_list, ACL):
+            acl_list = [acl_list.get_acl_dict()]
+
+        data = {
+            "acl": json.dumps(acl_list),
         }
+        logger.debug(f"set acl data: {data}")
         res = requests.put(
             url,
             timeout=kwargs.pop("timeout", 30),
-            data={"acl": json.dumps(acl)},
+            data=data,
             auth=HTTPBasicAuth(self.user, self.password),
         )
         if res.ok:
-            return res.json()
+            return "ok"
         raise OcPyRequestException(
             "could not set ACL! ("
             + res.text
             + " ["
             + str(res.status_code)
             + "])"
+            + "acl: "
+            + json.dumps(acl_list)
         )
 
-    def add_to_acl(self, acl: Union[ACL, List[ACL]], **kwargs):
+    def add_to_acl(self, acl: Union[ACL, List[ACL]]):
         logger.debug(f"adding to acl: {acl}")
+        actual_acl = self.get_acl()
         if not isinstance(acl, List):
             acl = [acl]
-        for a in acl:
-            url = (
-                self.base_url
-                + "/acl/"
-                + a.get_action().get_action_string().lower()
-            )
-            res = requests.post(
-                url,
-                timeout=kwargs.pop("timeout", 30),
-                data={"role": a.get_role(), "allow": a.allow},
-                auth=HTTPBasicAuth(self.user, self.password),
-            )
-            if not res.ok:
-                raise OcPyRequestException(
-                    "could not add to ACL! ("
-                    + res.text
-                    + " ["
-                    + str(res.status_code)
-                    + "])",
-                    res.status_code,
-                )
-        return "ok"
+        for n_a in acl:
+            found = False
+            for a_a in actual_acl:
+                if a_a == n_a:
+                    found = True
+                    break
+            if not found:
+                actual_acl.append(n_a)
 
-    def remove_from_acl(self, acl: ACL, **kwargs):
-        if isinstance(acl, List):
-            for a in acl:
-                url = (
-                    self.base_url
-                    + "/acl/"
-                    + a.get_action().get_action_string().lower().lower()
-                    + "/"
-                    + a.get_role()
-                )
-                res = requests.delete(
-                    url,
-                    timeout=kwargs.pop("timeout", 30),
-                    auth=HTTPBasicAuth(self.user, self.password),
-                )
-                if not res.ok:
-                    raise OcPyRequestException(
-                        "could not delete ACL! ("
-                        + res.text
-                        + " ["
-                        + str(res.status_code)
-                        + "])"
-                    )
-        else:
-            url = (
-                self.base_url
-                + "/acl/"
-                + acl.get_action().get_action_string().lower()
-                + "/"
-                + acl.get_role()
-            )
-            res = requests.delete(
-                url, timeout=kwargs.pop("timeout", 30), auth=HTTPBasicAuth(self.user, self.password)
-            )
-            if not res.ok:
-                raise OcPyRequestException(
-                    "could not delete ACL! ("
-                    + res.text
-                    + " ["
-                    + str(res.status_code)
-                    + "])"
-                )
-        return "ok"
+        return self.set_acl(actual_acl)
+
+    def remove_from_acl(self, acl: ACL):
+        actual_acl = self.get_acl()
+
+        if not isinstance(acl, List):
+            acl = [acl]
+        for aa in actual_acl:
+            for na in acl:
+                if aa == na:
+                    actual_acl.remove(na)
+        return self.set_acl(actual_acl)
 
     def get_metadata(self, raw=False, force_update=False, **kwargs):
         if "metadata" in self.data and not force_update:
             return self.data["metadata"]
         url = self.base_url + "/metadata"
-        res = requests.get(url, timeout=kwargs.pop("timeout", 30), auth=HTTPBasicAuth(self.user, self.password))
+        res = requests.get(
+            url,
+            timeout=kwargs.pop("timeout", 30),
+            auth=HTTPBasicAuth(self.user, self.password),
+        )
         if res.ok:
             if raw:
                 return res.json()
@@ -248,11 +214,7 @@ class Event:
                 # merge metadata
                 meta_result = {
                     **meta_result,
-                    **{
-                        m["id"]: m["value"]
-                        for m in r["fields"]
-                        if "value" in m
-                    },
+                    **{m["id"]: m["value"] for m in r["fields"] if "value" in m},
                 }
             return meta_result
         return None
@@ -279,15 +241,15 @@ class Event:
     def get_publications(self, force_update=False, **kwargs) -> List[Publication]:
         if "publications" in self.data and not force_update:
             return [
-                Publication(
-                    self.user, self.password, self.base_url + "/" + p["id"], p
-                )
+                Publication(self.user, self.password, self.base_url + "/" + p["id"], p)
                 for p in self.data["publications"]
             ]
         results = []
         url = self.base_url + "/publications"
         res = requests.get(
-            url, timeout=kwargs.pop("timeout", 30), auth=HTTPBasicAuth(self.user, self.password)
+            url,
+            timeout=kwargs.pop("timeout", 30),
+            auth=HTTPBasicAuth(self.user, self.password),
         )
         if res.ok:
             for p in res.json():
@@ -304,7 +266,9 @@ class Event:
     def get_publication(self, publication_id, **kwargs) -> Optional[Publication]:
         url = self.base_url + "/publications/" + publication_id
         res = requests.get(
-            url, timeout=kwargs.pop("timeout", 30), auth=HTTPBasicAuth(self.user, self.password)
+            url,
+            timeout=kwargs.pop("timeout", 30),
+            auth=HTTPBasicAuth(self.user, self.password),
         )
         if res.ok:
             p = res.json()
@@ -313,9 +277,7 @@ class Event:
             )
         return None
 
-    def get_publication_by_channel(
-        self, channel_name
-    ) -> Optional[Publication]:
+    def get_publication_by_channel(self, channel_name) -> Optional[Publication]:
         publications = self.get_publications()
         for p in publications:
             if p.get_channel() == channel_name:
@@ -335,24 +297,46 @@ class Event:
         if "scheduling" in self.data and not force_update:
             return self.data["scheduling"]
         url = self.base_url + "/scheduling"
-        res = requests.get(url, timeout=kwargs.pop("timeout", 30), auth=HTTPBasicAuth(self.user, self.password))
+        res = requests.get(
+            url,
+            timeout=kwargs.pop("timeout", 30),
+            auth=HTTPBasicAuth(self.user, self.password),
+        )
         if res.ok and res.status_code != 204:
             return Scheduling(**res.json())
         return None
 
+    def set_scheduling(self, scheduling: Scheduling, allow_conflict=False, **kwargs):
+        url = self.base_url + "/scheduling"
+        # put request using application/x-ww-form-urlencoded
+        form_data = {
+            "scheduling": json.dumps(scheduling.get_scheduling_dict()),
+            "allowConflict": allow_conflict,
+        }
+
+        res = requests.put(
+            url,
+            timeout=kwargs.pop("timeout", 30),
+            auth=HTTPBasicAuth(self.user, self.password),
+            data=form_data,
+            **kwargs,
+        )
+        if res.ok:
+            self.data["scheduling"] = Scheduling(**scheduling.get_scheduling_dict())
+            return None
+        raise OcPyRequestException("could not set event scheduling!", response=res)
+
     def delete(self, **kwargs):
         url = self.base_url
         res = requests.delete(
-            url, timeout=kwargs.pop("timeout", 30), auth=HTTPBasicAuth(self.user, self.password)
+            url,
+            timeout=kwargs.pop("timeout", 30),
+            auth=HTTPBasicAuth(self.user, self.password),
         )
         if res.ok:
             return "ok"
         raise OcPyRequestException(
-            "Could not delete event ("
-            + self.get_identifier()
-            + ")! ("
-            + res.text
-            + ")"
+            "Could not delete event (" + self.get_identifier() + ")! (" + res.text + ")"
         )
 
 
@@ -434,9 +418,7 @@ class EventsApi(OpenCastBaseApiClient):
                         res.status_code, res.content
                     )
                 )
-            raise OcPyRequestException(
-                res.content.decode("utf-8"), response=res
-            )
+            raise OcPyRequestException(res.content.decode("utf-8"), response=res)
         return results
 
     def get_all_events(
@@ -450,13 +432,42 @@ class EventsApi(OpenCastBaseApiClient):
         events_filter=None,
         batch_size=100,
         print_progress=False,
-    ) -> List[Event]:
-        result = []
+        initial_offset=0,
+        generator=False,
+    ) -> List[Event] | Generator[Event, None, None]:
+        events_generator = self.get_all_events_generator(
+            sign=sign,
+            with_acl=with_acl,
+            with_metadata=with_metadata,
+            with_scheduling=with_scheduling,
+            with_publications=with_publications,
+            sort=sort,
+            events_filter=events_filter,
+            batch_size=batch_size,
+            print_progress=print_progress,
+            initial_offset=initial_offset,
+        )
+        if generator:
+            return events_generator
+        return list(events_generator)
+
+    def get_all_events_generator(
+        self,
+        sign=True,
+        with_acl=False,
+        with_metadata=False,
+        with_scheduling=False,
+        with_publications=False,
+        sort=None,
+        events_filter=None,
+        batch_size=100,
+        print_progress=False,
+        initial_offset=0,
+    ) -> Generator[Event, None, None]:
+        count = initial_offset
         while True:
             if print_progress:
-                print(
-                    f"requesting {batch_size} starting from offset {len(result)}"
-                )
+                logger.info(f"requesting {batch_size} starting from offset {count}")
             res = self.get_events(
                 sign=sign,
                 with_acl=with_acl,
@@ -466,13 +477,18 @@ class EventsApi(OpenCastBaseApiClient):
                 with_publications=with_publications,
                 events_filter=events_filter,
                 limit=batch_size,
-                offset=len(result),
+                offset=count,
             )
             if res is None or len(res) <= 0:
-                print(f"DONE; got {len(result)} events")
+                logger.debug(
+                    f"DONE; got 0 resuts; retrieved a total of {len(res)} events"
+                )
                 break
-            result.extend(res)
-        return result
+            logger.debug(f"got {len(res)} events")
+            count += len(res)
+            for r in res:
+                yield r
+        return
 
     def get_event(
         self,
@@ -530,9 +546,7 @@ class EventsApi(OpenCastBaseApiClient):
         )
         if res.ok:
             return res.json()
-        raise OcPyRequestException(
-            "could not get event metadata!", response=res
-        )
+        raise OcPyRequestException("could not get event metadata!", response=res)
 
     def get_event_publications(self, event_id, **kwargs) -> PublicationList:
         res = requests.get(
@@ -543,9 +557,7 @@ class EventsApi(OpenCastBaseApiClient):
         )
         if res.ok:
             return PublicationList(res.json())
-        raise OcPyRequestException(
-            "could not get event publications!", response=res
-        )
+        raise OcPyRequestException("could not get event publications!", response=res)
 
     def get_event_scheduling(self, event_id, **kwargs) -> dict:
         res = requests.get(
@@ -556,9 +568,26 @@ class EventsApi(OpenCastBaseApiClient):
         )
         if res.ok:
             return res.json()
-        raise OcPyRequestException(
-            "could not get event scheduling!", response=res
+        raise OcPyRequestException("could not get event scheduling!", response=res)
+
+    def set_event_scheduling(
+        self, event_id, scheduling: Scheduling, allow_conflict=False, **kwargs
+    ) -> None:
+        form_data = {
+            "scheduling": json.dumps(scheduling.get_scheduling_dict()),
+            "allowConflict": allow_conflict,
+        }
+
+        res = requests.put(
+            self.base_url + f"/{event_id}/scheduling",
+            timeout=kwargs.pop("timeout", 30),
+            auth=HTTPBasicAuth(self.user, self.password),
+            data=form_data,
+            **kwargs,
         )
+        if res.ok:
+            return None
+        raise OcPyRequestException("could not set event scheduling!", response=res)
 
     def create_event(
         self, acl, metadata, processing, presenter_file, **kwargs
@@ -658,7 +687,9 @@ class EventsApi(OpenCastBaseApiClient):
         logger.error(f"status code: {res.status_code}", res.text)
         info = ""
         if res.status_code == 409:
-            info = "Additional info: (409) Conflict: (probably) conflicting events found)"
+            info = (
+                "Additional info: (409) Conflict: (probably) conflicting events found)"
+            )
         raise OcPyRequestException(
             f"could not create event! ({res.text}) - {info}",
             response=res,
@@ -674,9 +705,7 @@ class EventsApi(OpenCastBaseApiClient):
         )
         if res.ok:
             return "ok"
-        raise OcPyRequestException(
-            "could not delete event! (" + res.text + ")"
-        )
+        raise OcPyRequestException("could not delete event! (" + res.text + ")")
 
     def get_events_part_of(self, series, limit=100, **kwargs) -> List[Event]:
         if not isinstance(series, str):
@@ -687,6 +716,58 @@ class EventsApi(OpenCastBaseApiClient):
             **kwargs,
         )
 
+    def add_track(
+        self,
+        event_id: str,
+        flavor: str,
+        track_file,
+        tags: List[str],
+        overwrite_existing=False,
+        **kwargs,
+    ) -> str:
+        """Adds a track to an event using a POST request and form data.
+        Args:
+            event_id (str): The ID of the event to which the track will be added.
+            flavor (str): The flavor of the track (e.g., "captions/test").
+            track_file (str): The path to the track file to be uploaded.
+            tags (List[str]): A list of tags to be associated with the track.
+            overwrite_existing (bool): Whether to overwrite an existing track with the same flavor. - Sadly this seems to not work
+            yet, so you have to delete the track first if you want to overwrite it.
+            **kwargs: Additional keyword arguments for the request.
+        """
+
+        url = f"{self.base_url}/{event_id}/track"
+        files = {"track": (os.path.basename(track_file), open(track_file, "rb"))}
+        data = {
+            "eventId": event_id,
+            "overwriteExisting": overwrite_existing,
+            "flavor": flavor,
+            "tags": ",".join(tags) if tags else "",
+        }
+        try:
+            res = requests.post(
+                url,
+                timeout=kwargs.pop("timeout", 30),
+                auth=HTTPBasicAuth(self.user, self.password),
+                files=files,
+                data=data,
+                **kwargs,
+            )
+            if res.ok:
+                return True
+            raise OcPyRequestException(
+                f"Could not add track to event {event_id}! ({res.text})",
+                response=res,
+            )
+        except FileNotFoundError as e:
+            raise OcPyRequestException(
+                f"File not found: {track_file}. Please check the file path."
+            ) from e
+        except requests.RequestException as e:
+            raise OcPyRequestException(
+                f"An error occurred while adding the track: {str(e)}"
+            ) from e
+
     class Filter:
         def __init__(self):
             self.filter_string = None
@@ -732,130 +813,3 @@ class EventsApi(OpenCastBaseApiClient):
 
         def get_filter_string(self):
             return self.filter_string
-
-
-def create_test_event(ev_api):
-    test_meta = [
-        {
-            "fields": [
-                {
-                    "id": "title",
-                    "label": "EVENTS.EVENTS.DETAILS.METADATA.TITLE",
-                    "type": "text",
-                    "value": "Testevent",
-                }
-            ],
-            "flavor": "kit/episode",
-        }
-    ]
-
-    test_acl = ACL.get_read_write_acls("ROLE_TEST_USER")
-    # test_acl = []
-
-    processing_info = {
-        "configuration": {
-            "comment": "false",
-            "publishLive": "false",
-            "publishToMediaModule": "true",
-            "publishToOaiPmh": "true",
-        },
-        "workflow": "fast",
-    }
-    # "workflow": "import"
-    # "workflow": "fast"}
-
-    test_video_file = os.path.abspath("../../../video_test_512kb.mp4")
-    return ev_api.create_event(
-        test_acl, test_meta, processing_info, test_video_file
-    )
-
-
-def main():
-    # api = EventsApi(server_url="https://opencast-test.bibliothek.kit.edu", user="admin_api_user", password="abcxyz")
-    # api = EventsApi(server_url="https://opencast-qa.bibliothek.kit.edu", user="api_user", password="123abcxyz")
-    api = EventsApi(
-        server_url="https://oc9-dev-admin.bibliothek.kit.edu",
-        user="admin",
-        password="password",
-    )
-    # evs = api.get_events(events_filter=EventsApi.Filter())
-    evs = api.get_events(limit=10)
-    print("There are {} events.".format(len(evs)))
-
-    # create_test_event()
-
-    # print(create_test_event(api))
-    # api = EventsApi(server_url="http://opencast-dev.bibliothek.kit.edu:8080")
-    # evs = api.get_events(limit=19, offset=0, with_publications=True, with_metadata=True)
-    # evs = api.get_events(limit=19, offset=0, with_publications=False, with_metadata=False)
-    # evs = api.get_all_events()
-    print("There are {} events.".format(len(evs)))
-    for e in evs:
-        # pprint(e.get_metadata(True))
-        if e.get_metadata(False) is None:
-            print("NO META!!")
-            print(e)
-            print("NO META!!")
-
-        else:
-            print(e)
-            # print(e.delete_metadata())
-            # print(e.get_metadata())
-        # print(e.get_metadata(False)["title"])
-        # if e.get_metadata(False)["title"] == "test3":
-        #    print("found!!!")
-        # e.delete()
-        # pprint(e.get_acl())
-        # e.add_to_acl(ACL.get_read_write_acls("ROLE_FAKE_USER"))
-        # pprint(e.get_acl())
-        # print(e.delete())
-    print(evs)
-    print(len(evs))
-    acl = api.get_events(
-        limit=1, offset=0, with_publications=True, with_metadata=True
-    )[0].get_acl()
-
-    print(
-        api.get_events(
-            limit=1, offset=0, with_publications=True, with_metadata=True
-        )[0].get_acl()
-    )
-    try:
-        print(
-            api.get_events(
-                limit=1, offset=146, with_publications=True, with_metadata=True
-            )
-        )
-    except OcPyRequestException as e:
-        print(e.is_opencast_error())
-    # exit()
-    # evs = api.get_events()
-    default_acl = evs[0].get_acl()
-    print(default_acl)
-
-    pprint(evs[0].get_metadata(True))
-
-    exit()
-    # api = EventsApi()
-    # print(api.get_events_part_of("911ab6f7-67f9-48b5-b834-8855656c2991"))
-
-    ev = api.get_events(limit=1, offset=0)[0]
-    evs = api.get_events(limit=1, offset=14)
-    evs = api.get_events(limit=100)
-    # evs = api.get_events(filter=EventsApi.Filter().set_location_filter("Tulla_SMP"))
-    # evs = api.get_events(filter=EventsApi.Filter().set_series_filter("6b7e27a0-aec4-400c-88a4-8418081c0d30"))
-    # evs = api.get_events(limit=100, offset=14)
-    # evs.append(ev)
-    for e in evs:
-        # print(e.())
-        pprint(e)
-
-        pprint(e.get_metadata())
-        pprint(e.get_acl())
-        pprint(e.get_publications())
-        pprint(e.get_scheduling())
-    print(len(evs))
-
-
-if __name__ == "__main__":
-    main()
