@@ -9,7 +9,7 @@ from ocpy.api.api_client import OpenCastBaseApiClient
 class OrgApi(OpenCastBaseApiClient):
     def __init__(self, user=None, password=None, server_url=None, **_kwargs):
         super().__init__(user, password, server_url)
-        self.base_url = server_url + "/org"
+        self.base_url = self.server_url + "/org"
 
     def get_all_orgs(self, **kwargs):
         res = requests.get(
